import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import generated.*;

import java.util.List;

public class JAXBSample {

    public static void main(String args[]) throws javax.xml.bind.JAXBException{
 
        try {
            //Create JAXBContext using package/namespace
            JAXBContext jaxbContext = JAXBContext.newInstance("generated");
            
            //Create ObjectFactory for each schema namespace
            ObjectFactory objectFactory= new ObjectFactory();

            //Prepare to test Marshalling.  Initialize Data, then marshal it to a file.
            CatalogData catalogData = initData(objectFactory);
            testMarshal(jaxbContext, catalogData);

            //Unmarshal the Object you just marshalled and print it out for viewing.
            testUnMarshal(jaxbContext);
        } catch(javax.xml.bind.JAXBException  e) {
        }
    }

/**
     * Test the marshalling of a JAXB Object into a file.
     * 
     * 
     * 
     * @param jaxbContext     JAXBContext required to create the marshaller from.
     * @param catalogData     JAXB Object to marshal.
     */
    public static void testMarshal(JAXBContext jaxbContext, CatalogData catalogData) {

        try {
            //Create file you'd like to use for output.
            java.io.OutputStream os = new java.io.FileOutputStream( "cdDoc.xml" );

            //Create marhsaller
            Marshaller marshaller = jaxbContext.createMarshaller();

            //Set property to make the xml printed out human readable.
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);

            //Marshall object into file created above.
            marshaller.marshal(catalogData, os);

        } catch(Exception e) {
        }
    }
/**
     * Test the unmarshalling of an xml document into
     * JAXB Objects
     * 
     * 
     * @param jaxbContext     JAXBContext required to create unmarshaller.
     */

    public static void testUnMarshal(JAXBContext jaxbContext) {

        
        try {
            //Create unmarshaller.
            Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();

            //unmarshal into catalogData the contents of the xml document cdDoc.xml
            CatalogData catalogData = (CatalogData) unmarshaller.unmarshal
            (new java.io.FileInputStream( "cdDoc.xml" )); 

            //Print out the contents, and verify objects were unmarshalled correctly.
            prettyPrint(catalogData);

        } catch(Exception e) {
        }
    }
    
    /**
     * Prepare data for marshalling.
     * 
     * @param objFact ObjectFactory used to create elements/types defined in schema.
     * @return Fully initialized catalog data.
     */
    public static CatalogData initData(ObjectFactory objFact) {

        // Note.. you can create JAXB Objects thru the objects constructors or
        // thru the ObjectFactory's create methods.

        // Using Object's constructor to create BookData.  Bookdata is used build
        // initialize CatalogData further below.
        Bookdata bookData1 = new Bookdata();
        bookData1 = initBook("Gambardella, Matthew", "XML Developer's Guide","Computer", (float)44.01,
                      "An in-depth look at creating applications with  XML.", "bk101", bookData1);

        Bookdata bookData2 = new Bookdata();;
        bookData2 = initBook("J.R.R. Tolkien", "The Hobbit", "fiction", (float)23.98, 
                       "A hobbits treacherous journey to destroy a ring", "bk202", bookData2);

        // Use ObjectFactory to create CatalogData. CatalogData is a root element.
        CatalogData catalogData= objFact.createCatalogData();


        //getBooks returns a list.  Add books above to the list.
        catalogData.getBooks().add(bookData1);
        catalogData.getBooks().add(bookData2);
        return catalogData;
    }
/**
     * Prepare data for marshalling.  Initialize the Bookdata
     * 
     * @return Fully initialized Bookdata.
     */
    public static Bookdata initBook(String author, String title, 
                                    String genre, float price, 
                                    String desc, String id, Bookdata bookData) {
        bookData.setAuthor(author);
        // . . .
        bookData.setId(id);
        return bookData;
    }


    /**
     * Pretty print a Catalog Data object out to System.out.
* 
     * @param catalogData     CatalogData object you'd like to pretty print.
     */
    public static void prettyPrint(CatalogData catalogData) {

        List books = catalogData.getBooks();
        for(int i=0; i< books.size(); i++) {
            Bookdata bookData = (Bookdata) books.get(i);
            System.out.println("CATALOG:  author= " +  bookData.getAuthor());
            // . . .
            System.out.println("CATALOG:  id= " +  bookData.getId());
        }
    }
}

